/*
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *
 *                  L'mulateur Thomson TO8
 *
 *  Copyright (C) 1997-2001 Gilles Ftis, Eric Botcazou, Alexandre Pukall,
 *                          Jrmie Guillaume
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : alleg/gui.c
 *  Version    : 1.7
 *  Cr par   : Gilles Ftis 1998
 *  Modifi par: Jrmie GUILLAUME alias "JnO" 1998
 *               Eric Botcazou 24/09/2001
 *
 *  Panneau de contrle de l'mulateur.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <string.h>
   #include <dir.h>
   #include <allegro.h>
#endif

#include "alleg/sound.h"
#include "alleg/main.h"
#include "to8.h"


static char title[35];  /* titre commun des botes de dialogue */


/*****************/
/**** Message ****/
/*****************/

static char mesg[35];

static DIALOG mesgdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    mesg },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,      10,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define BUTTON_FIX 6

static void PopupMessage(const char message[])
{
    mesgdial[0].w=strlen(message)*8+16;
    mesgdial[1].x=mesgdial[0].x+mesgdial[0].w/2;
    mesgdial[2].x=mesgdial[1].x-(mesgdial[2].w+BUTTON_FIX)/2;
    
    strcpy(mesg, message);

    centre_dialog(mesgdial);

    popup_dialog(mesgdial, 2);
}


/******************/
/**** Question ****/
/******************/

static char quest[35];

static DIALOG questdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    quest },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&Oui" },
{ d_button_proc,    130,   40,   32,   16,     0,     0, 'n',    D_EXIT,  0,    0,    "&Non" },
{ d_yield_proc,      10,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define FOCUS_YES  2
#define FOCUS_NO   3

static int PopupQuestion(const char question[], int focus)
{
    int esp;
    
    questdial[0].w=strlen(question)*8+16;
    questdial[1].x=questdial[0].x+questdial[0].w/2;

    esp=(questdial[0].w-2*(questdial[2].w+BUTTON_FIX))/3;
    questdial[2].x=questdial[0].x+esp;
    questdial[3].x=questdial[2].x+(questdial[2].w+BUTTON_FIX)+esp;
    
    strcpy(quest, question);

    centre_dialog(questdial);
    
    return (popup_dialog(questdial, focus) == 2 ? TRUE : FALSE);
}


/*************************/
/**** Menu imprimante ****/
/*************************/

static BITMAP *nlq_led, *on_led, *online_led;

static DIALOG printerdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Imprimante matricielle PR 90-612" },
{ d_box_proc,         40,  50,  100,  110,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,       90,  55,    0,    0,     0,     0,    0,    0,       0,    0,    "NLQ/Failure" },
{ d_box_proc,         60,  65,   60,   16,     0,     0,    0,    0,       0,    0,    NULL },
{ d_bitmap_proc,      61,  66,   58,   14,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,       90,  90,    0,    0,     0,     0,    0,    0,       0,    0,    "On" },
{ d_box_proc,         60, 100,   60,   16,     0,     0,    0,    0,       0,    0,    NULL },
{ d_bitmap_proc,      61, 101,   58,   14,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,       90, 125,    0,    0,     0,     0,    0,    0,       0,    0,    "On-Line" },
{ d_box_proc,         60, 135,   60,   16,     0,     0,    0,    0,       0,    0,    NULL },
{ d_bitmap_proc,      61, 136,   58,   14,     0,     0,    0,    0,       0,    0,    NULL },
{ d_text_proc,       160,  50,    0,    0,     0,     0,    0,    0,       0,    0,    "Commandes:" }, 
{ d_button_proc,     160,  65,  120,   16,     0,     0,    0,    D_EXIT,  0,    0,    "On-Line" },
{ d_button_proc,     160,  85,  120,   16,     0,     0,    0,    D_EXIT,  0,    0,    "Line Feed" },
{ d_button_proc,     160, 105,  120,   16,     0,     0,    0,    D_EXIT,  0,    0,    "Form Feed/NLQ" },
{ d_text_proc,       175, 135,    0,    0,     0,     0,    0,    0,       0,    0,    "Sortie" },
{ d_check_proc,      175, 145,   90,   14,     0,     0,    0,    D_SELECTED,0,  0,    "graphique" },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,    0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

static void menuprinter(void)
{
    static int first = 1;
    static int leds[3];

    if (first)
    {
        nlq_led = create_bitmap(58, 14);
        printerdial[6].dp = nlq_led;

        on_led = create_bitmap(58, 14);
        printerdial[9].dp = on_led;

        online_led = create_bitmap(58, 14);
        printerdial[12].dp = online_led;

        to8_PrinterEnableGraphicMode(TRUE);
        first = 0;
    }

    centre_dialog(printerdial);
    clear_keybuf();

    while (TRUE)
    {
        /* lit l'tat de l'imprimante */
        to8_PrinterGetState(leds);
        clear_to_color(nlq_led, leds[0] ? makecol(255, 0, 0) : gui_bg_color);
        clear_to_color(on_led, leds[1] ? makecol(255, 255, 0) : gui_bg_color);
        clear_to_color(online_led, leds[2] ? makecol(0, 255, 0) : gui_bg_color);

        switch (popup_dialog(printerdial, 19))
        {
            case 14:
                to8_PrinterSendCommand(TO8_PRINTER_ONLINE);
                break;

            case 15:
                to8_PrinterSendCommand(TO8_PRINTER_LINE_FEED);
                break;

            case 16:
                to8_PrinterSendCommand(TO8_PRINTER_FORM_FEED);
                break;

            case 19:
                to8_PrinterEnableGraphicMode(printerdial[18].flags & D_SELECTED);
                return;
        }
    }
}


/*************************/
/**** Menu disquettes ****/
/*************************/

#define LABEL_LENGTH     20
static char disk_label[4][LABEL_LENGTH+1];

static DIALOG diskdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 20,   10,  280,  180,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,     160,   20,    0,    0,     0,     0,   0,    0,       0,    0,    title },
{ d_ctext_proc,     160,   30,    0,    0,     0,     0,   0,    0,       0,    0,    "Lecteurs de disquettes" },
  /* disk 0 */
{ d_text_proc,       30,   54,    0,    0,     0,     0,   0,    0,       0,    0,    "0:" },
{ d_textbox_proc,    47,   50,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[0] },
{ d_button_proc,    220,   50,   30,   16,     0,     0, '0',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   50,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 1 */
{ d_text_proc,       30,   78,    0,    0,     0,     0,   0,    0,       0,    0,    "1:" },
{ d_textbox_proc,    47,   74,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[1] },
{ d_button_proc,    220,   74,   30,   16,     0,     0, '1',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   74,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 2 */
{ d_text_proc,       30,  102,    0,    0,     0,     0,   0,    0,       0,    0,    "2:" },
{ d_textbox_proc,    47,   98,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[2] },
{ d_button_proc,    220,   98,   30,   16,     0,     0, '2',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   98,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 3 */
{ d_text_proc,       30,  126,    0,    0,     0,     0,   0,    0,       0,    0,    "3:" },
{ d_textbox_proc,    47,  122,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[3] },
{ d_button_proc,    220,  122,   30,   16,     0,     0, '3',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,  122,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* direct disk */
{ d_button_proc,     30,  146,  250,   16,     0,     0, 'd',    D_EXIT,  0,    0,    "Accs &direct - lecteur 0:" },
{ d_text_proc,      255,   40,    0,    0,     0,     0,   0,    0,       0,    0,    "prot." },
{ d_button_proc,     30,  170,  126,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,      20,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};


static void menudisk(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int drive, ret, ret2;
    
    if (first)
    {
        /* la premire fois on tente d'ouvrir le rpertoire par dfaut */
        if (file_exists("./disks", FA_DIREC, NULL))
	    strcpy(filename,"./disks/");
	
        centre_dialog(diskdial);

        for (drive=0; drive<4; drive++)
        {
            strncpy(disk_label[drive], get_filename(to8_GetDiskFilename(drive)), LABEL_LENGTH);
            if (disk_label[drive][0] == '\0')
                strncpy(disk_label[drive], "aucune disquette", LABEL_LENGTH);
        }

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        ret=popup_dialog(diskdial, 21);
        
        switch (ret)
        {
            case  5:
            case  9:
            case 13:
            case 17:
                drive=(ret-5)/4;
            
                if (file_select_ex("Choisissez votre disquette:", filename, "sap",
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    ret2=to8_LoadDisk(drive, filename);

                    if (ret2 == TO8_ERROR)
                        PopupMessage(to8_error_msg);
                    else
                    {
                        strncpy(disk_label[drive], get_filename(filename), LABEL_LENGTH);

                        if ((ret2==TO8_READ_ONLY) && !(diskdial[ret+1].d2))
                        {
                            PopupMessage("Attention: criture impossible.");
                            diskdial[ret+1].flags|=D_SELECTED;
                            diskdial[ret+1].d2=1;
                        }
                    }
                }
                break;

            case  6:
            case 10:
            case 14:
            case 18:
                drive=(ret-6)/4;
            
                if (diskdial[ret].d2)
                {
                    if (to8_SetDiskMode(drive, TO8_READ_WRITE)==TO8_READ_ONLY)
                    {
                        PopupMessage("Ecriture impossible sur ce support.");
                        diskdial[ret].flags|=D_SELECTED;
                        diskdial[ret].d2=1;
                    }
                    else
                    {
                        diskdial[ret].flags&=~D_SELECTED;
                        diskdial[ret].d2=0;
                    }
                }
                else
                {
                    to8_SetDiskMode(drive, TO8_READ_ONLY);
                    diskdial[ret].flags|=D_SELECTED;
                    diskdial[ret].d2=1;
                }
                break;

            case 19:
                strncpy(disk_label[0], "accs direct", LABEL_LENGTH);

                if (to8_DirectSetDrive(0) == TO8_READ_ONLY)
                {
                    diskdial[6].flags|=D_SELECTED;
                    diskdial[6].d2=1;
                }
                else
                {
                    diskdial[6].flags&=~D_SELECTED;
                    diskdial[6].d2=0;
                }
                break;

            case 21:
                return;

            default:
        }
    }
}


/************************/
/**** Menu cassettes ****/
/************************/

static char k7_label[LABEL_LENGTH+1];

static DIALOG k7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cassettes" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "k7" },
{ d_textbox_proc,     47,  50,  167,   16,     0,     0,    0,    0,       0,    0,    k7_label },
{ d_button_proc,     220,  50,   30,   16,     0,     0,  'k',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,      260,  50,   15,   15,     0,     0,    0,    D_EXIT|D_SELECTED,0,1,"" },
{ d_text_proc,       255,  40,    0,    0,     0,     0,   0,     0,       0,    0,    "prot." },
{ d_button_proc,      30,  74,  250,   16,     0,     0,  'r',    D_EXIT,  0,    0,    "&Rembobiner la cassette" },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,    0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

static void menuk7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int ret;

    if (first)
    {
        /* la premire fois on tente d'ouvrir le rpertoire par dfaut */
    	if (file_exists("./k7", FA_DIREC, NULL))
	    strcpy(filename,"./k7/");

        centre_dialog(k7dial);

        strncpy(k7_label, get_filename(to8_GetK7Filename()), LABEL_LENGTH);
        if (k7_label[0] == '\0')
            strncpy(k7_label, "aucune cassette", LABEL_LENGTH);

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        ret=popup_dialog(k7dial, 9);
        
        switch (ret)
        {
            case 5:
                if (file_select_ex("Choisissez votre cassette:", filename, "k7", 
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    ret=to8_LoadK7(filename);

                    if (ret == TO8_ERROR)
                        PopupMessage(to8_error_msg);
                    else
                    {
                        strncpy(k7_label, get_filename(filename), LABEL_LENGTH);

                        if ((ret==TO8_READ_ONLY) && !(k7dial[6].d2))
                        {
                            PopupMessage("Attention: criture impossible.");
                            k7dial[6].flags|=D_SELECTED;
                            k7dial[6].d2=1;
                        }
                    }
                }
                break;

            case 6:
                if (k7dial[6].d2)
                {
                    if (to8_SetK7Mode(TO8_READ_WRITE)==TO8_READ_ONLY)
                    {
                        PopupMessage("Ecriture impossible sur ce support.");
                        k7dial[6].flags|=D_SELECTED;
                        k7dial[6].d2=1;
                    }
                    else
                    {
                        k7dial[6].flags&=~D_SELECTED;
                        k7dial[6].d2=0;
                    }
                }
                else
                {
                    to8_SetK7Mode(TO8_READ_ONLY);
                    k7dial[6].flags|=D_SELECTED;
                    k7dial[6].d2=1;
                }
                break;

            case 8:
                to8_RewindK7();
                break;

            case 9:
                return;
        }
    }
}


/*************************/
/**** Menu cartouches ****/
/*************************/

static char m7_label[TO8_MEMO7_LABEL_LENGTH+1];

static DIALOG m7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cartouches" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "m7" },
{ d_textbox_proc,     47,  50,  208,   16,     0,     0,    0,    0,       0,    0,    m7_label },
{ d_button_proc,     260,  50,   30,   16,     0,     0,  'm',    D_EXIT,  0,    0,    "..." },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,    0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

static void menumemo7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int ret;

    if (first)
    {
        /* la premire fois on tente d'ouvrir le rpertoire par dfaut */
	if (file_exists("./memo7", FA_DIREC, NULL))
	    strcpy(filename,"./memo7/");
	
        centre_dialog(m7dial);

        strncpy(m7_label, to8_GetMemo7Label(), TO8_MEMO7_LABEL_LENGTH);
        if (m7_label[0] == '\0')
            strncpy(m7_label, "aucune cartouche", TO8_MEMO7_LABEL_LENGTH);

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        ret=popup_dialog(m7dial, 6);

        switch (ret)
        {
            case 5:
                if (file_select_ex("Choisissez votre cartouche:", filename, "m7",
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    if (to8_LoadMemo7(filename) == TO8_ERROR)
                        PopupMessage(to8_error_msg);
                    else
                    {
                        strncpy(m7_label, to8_GetMemo7Label(), TO8_MEMO7_LABEL_LENGTH);
                        teo.command=COLD_RESET;
                    }   
                }
                break;

            case 6:
                return;
        }
    }
}


/*************************************/
/**** Menu commmandes et rglages ****/
/*************************************/

static DIALOG commdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)    (fg)   (bg)   (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,   280,  180,     0,     0,      0,   0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,     0,    0,     0,     0,      0,   0,       0,    0,    title },
{ d_ctext_proc,      160,  30,     0,    0,     0,     0,      0,   0,       0,    0,    "Commandes et Rglages" },
{ d_button_proc,      30,  50,   260,   16,     0,     0,    'r',   D_EXIT,  0,    0,    "&Rinitialiser le TO8" },
{ d_button_proc,      30,  74,   260,   16,     0,     0,    'f',   D_EXIT,  0,    0,    "&Redmarrer  &froid le TO8" },
{ d_text_proc,        30, 100,     0,    0,     0,     0,      0,   0,       0,    0,    "Vitesse:" },
{ d_radio_proc,      105, 100,   126,    8,     0,     0,    'e',   0,       1,    0,    "&exacte" },
{ d_radio_proc,      105, 114,   126,    8,     0,     0,    'm',   0,       1,    0,    "&maximale" },
{ d_text_proc,        30, 138,     0,    0,     0,     0,      0,   0,       0,    0,    "Volume &sonore:" },
{ d_text_proc,       150, 152,     0,    0,     0,     0,      0,   0,       0,    0,    "bas         haut" },
{ d_slider_proc,     160, 135,   100,   15,     0,     0,    's',   0,     254,    0,    NULL},
{ d_button_proc,      30, 170,   126,   16,     0,     0,    'o',   D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,     0,    0,     0,     0,      0,   0,       0,    0,    NULL },
{ NULL,                0,   0,     0,    0,     0,     0,      0,   0,       0,    0,    NULL }
};

static void menucomm(void)
{
    if (teo.exact_speed)
    {
        commdial[6].flags=D_SELECTED;
        commdial[7].flags=0;
    }
    else
    {
        commdial[6].flags=0;
        commdial[7].flags=D_SELECTED;
    }

    if (teo.sound_enabled && teo.exact_speed)
        commdial[10].d2=GetVolume()-1;

    clear_keybuf();
    centre_dialog(commdial);

    switch (popup_dialog(commdial, 11))
    {
        case 3:
            teo.command=RESET;
            break;

        case 4:
            teo.command=COLD_RESET;
            break;
    }

    if (teo.sound_enabled && teo.exact_speed)
        SetVolume(commdial[10].d2+1);

    teo.exact_speed=(commdial[6].flags&D_SELECTED ? TRUE : FALSE);
}


/*****************************/
/**** Panneau de contrle ****/
/*****************************/ 

static DIALOG controldial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg) (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,   0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,   0,    0,       0,    0,    "Gilles Ftis - Eric Botcazou" },
{ d_ctext_proc,      160,  40,    0,    0,     0,     0,   0,    0,       0,    0,    "Alex Pukall - Jrmie Guillaume" },
{ d_button_proc,      30,  56,  260,   16,     0,     0,   'c',  D_EXIT,  0,    0,    "&Commandes et Rglages..." },
{ d_button_proc,      30,  78,  260,   16,     0,     0,   't',  D_EXIT,  0,    0,    "Lecteur de car&touches..." },
{ d_button_proc,      30, 100,  260,   16,     0,     0,   's',  D_EXIT,  0,    0,    "Lecteur de ca&ssettes..." },
{ d_button_proc,      30, 122,  260,   16,     0,     0,   'd',  D_EXIT,  0,    0,    "Lecteurs de &disquettes..." },
{ d_button_proc,      30, 144,  260,   16,     0,     0,   'i',  D_EXIT,  0,    0,    "&Imprimante matricielle..." },
{ d_button_proc,      30, 170,  126,   16,     0,     0,   'o',  D_EXIT,  0,    0,    "&OK" },
{ d_button_proc,     164, 170,  126,   16,     0,     0,   'q',  D_EXIT,  0,    0,    "&Quitter" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

void ControlPanel(void)
{
    /* we never read the key buffer, we have to clear it to avoid problems */
    clear_keybuf();
    centre_dialog(controldial);

    while (TRUE)
        switch (popup_dialog(controldial, 9))
        {
            case 4:
                menucomm();
                break;

            case 5:
                menumemo7();
                break;

            case 6:
                menuk7();
                break;

            case 7:
                menudisk();
                break;

            case 8:
                menuprinter();
                break;

            case 9:
                return;

            case 10:
                if (PopupQuestion("Voulez-vous vraiment quitter ?", FOCUS_NO))
                {
                    teo.command=QUIT;
                    return;
                }
        }
}



/* SetColorsGUI:
 *  Fixe les 3 couleurs de l'interface utilisateur.
 */
void SetColorsGUI(int fg_color, int bg_color, int bg_entry_color)
{
    set_dialog_color( mesgdial, fg_color, bg_color);
    set_dialog_color(questdial, fg_color, bg_color);
    
    set_dialog_color(printerdial, fg_color, bg_color);

    set_dialog_color(diskdial, fg_color, bg_color);
    diskdial[4].bg  = bg_entry_color;
    diskdial[8].bg  = bg_entry_color;
    diskdial[12].bg = bg_entry_color;
    diskdial[16].bg = bg_entry_color;
    
    set_dialog_color(k7dial, fg_color, bg_color);
    k7dial[4].bg = bg_entry_color;

    set_dialog_color(m7dial, fg_color, bg_color);
    m7dial[4].bg = bg_entry_color;
    
    set_dialog_color(commdial, fg_color, bg_color);
    set_dialog_color(controldial, fg_color, bg_color);

    /* Allegro GUI objects */
    gui_fg_color = fg_color;
    gui_bg_color = bg_color;
    gui_mg_color = bg_color;
}



/* InitGUI:
 *  Initialise le module interface utilisateur.
 */
void InitGUI(char version_name[], int direct_read_support)
{
    strcpy(title, version_name);

    if (!teo.sound_enabled)
        commdial[8].flags=commdial[9].flags=commdial[10].flags=D_DISABLED;

    if (!direct_read_support)
        diskdial[19].flags=D_HIDDEN;

    if (strlen(to8_GetMemo7Label()))
        strncpy(m7_label, to8_GetMemo7Label(), TO8_MEMO7_LABEL_LENGTH);
}

